import win32api, win32gui, win32con  # pip  install     pywin32  可能会出现问题_请看README_后端py.md




def wind_hide_taskbar_log(name_title="web - Google Chrome"):
    """ EXPLAIN : windows系统平台隐藏_任务栏图标,获取软件名称请借助  spy++
        # pip  install     pywin32  可能会出现问题_请看README_后端py.md
        EXAMPLE : print("result  :", wind_hide_taskbar_log("Postman"))  ## true/false       "Postman"/"web - Google Chrome"
    """
    handle_wind = win32gui.FindWindow(None, name_title)
    # print("handle_wind            :", handle_wind)
    result = win32gui.ShowWindow(handle_wind, 0)
    # print("result             :", result)
    if result >= 1:
        return True
    else:
        return False


if __name__ == '__main__':
    pass
    print("result  :", wind_hide_taskbar_log("Postman"))  ## true/false



# coding=utf-8
import os, io, sys, asyncio, requests, arrow, json, time
# import astmain as __


# coding=utf-8
import os, io, sys, asyncio, requests, arrow, json, time
# import astmain as __
import astmain_py777.astmain as __
from astmain_py777.astmain import print
__.print_hook = True
print = __.print

print("111                 :", 111)
print("111                 :", 111)
print("111                 :", 222)



import datetime

from .write_add import write_add
from ..colors import colors


def log_info(level="error", info="默认内容1", path=r"C:\log_all.txt", is_print=True, color=colors.cyan):
    """ EXPLAIN : 日志追加记录函数
        EXAMPLE :  __.fs.log_info(level="## 详情日志", info=str(sys.argv), path="aaa_dist_详情日志.md", is_print=True, color=__.colors.cyan)
    """

    content = level + "|||" + datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S") + "|||" + info
    if is_print: print(color, content, colors.end)
    write_add(path, content)


if __name__ == '__main__':
    log_info(info="我的错误", path=r"C:\log_all.txt")



import inspect


def execute_code(code_str, *args, **kwargs):
    """
    安全地执行给定的 Python 代码字符串,并返回函数的返回结果。

    Args:
        code_str (str): 要执行的 Python 代码字符串。
        *args: 要传递给函数的位置参数。
        **kwargs: 要传递给函数的关键字参数。

    Returns:
        tuple: 一个包含以下元素的元组:
            - bool: 代码执行是否成功。
            - Any: 如果代码块有返回语句,则返回相应的返回值;否则返回 None。
            - dict: 如果代码块定义了函数,则返回函数的返回结构;否则返回 None。
            - Any: 如果代码块定义了函数,则返回函数的执行结果;否则返回 None。
    """
    try:
        # 创建一个新的命名空间
        exec_namespace = {}

        # 使用 exec 函数执行代码字符串
        exec(code_str, exec_namespace, exec_namespace)

        # 检查是否有返回值
        success = True
        return_value = exec_namespace.get('return_value', None)

        # 检查是否定义了函数,并获取函数的返回结构和执行结果
        func_return_struct = None
        func_result = None
        for name, obj in exec_namespace.items():
            if callable(obj) and name != 'execute_code':
                try:
                    func_return_struct = inspect.getfullargspec(obj)
                    func_result = obj(*args, **kwargs)
                    break
                except TypeError:
                    # 如果对象不是函数,则跳过
                    pass

        # return success, return_value, func_return_struct, func_result
        return func_result

    except Exception as e:
        # 捕获并处理任何异常
        # print(f"执行代码时发生错误: {e}")
        return f"执行代码时发生错误: {e}"


if __name__ == '__main__':
    # 定义一个函数字符串
    func_def = """
def aaa(x, y=10):
    return x + y
    """

    # 执行函数字符串
    func_result = execute_code(func_def, 655, y=5)
    print(func_result)




from typing import Any, Dict


class to_obj_class:
    def __init__(self, data: Dict[str, Any]):
        self.__data = data

    def __getattr__(self, name: str) -> Any:
        try:
            return self.__data[name]
        except KeyError:
            raise AttributeError(f"'{type(self).__name__}' object has no attribute '{name}'")

    def __getitem__(self, key: str) -> Any:
        return self.__data[key]

    def __setattr__(self, name: str, value: Any) -> None:
        if name.startswith("_to_obj_class__"):
            super().__setattr__(name, value)
        else:
            self.__data[name] = value

    def __setitem__(self, key: str, value: Any) -> None:
        self.__data[key] = value

    def __dir__(self) -> list:
        return list(self.__data.keys()) + super().__dir__()


def to_obj(data: Dict[str, Any]) -> to_obj_class:
    """
    将字典转换为可以使用 obj["key"] 和 obj.key 访问的对象.

    Args:
        data (Dict[str, Any]): 需要转换的字典数据.

    Returns:
        DictAccessProxy: 转换后的对象.
    """
    return to_obj_class(data)


if __name__ == '__main__':
    bbb = dict(aaa=111, vvv=111)
    bbb = to_obj(bbb)
    print("111                 :", bbb.aaa)
    print("111                 :", bbb.vvv)
    # print("111                 :", bbb["aaa"])



# # coding=utf-8
# import os
# from ..colors import colors
#
# print_old = print
#
#
# def print_new(*args, **kwargs):
#     print_old(colors.yellow, *args, **kwargs)
#
#
# def print_hook(isok=False):
#     if isok == True:
#         os.environ['print_hook'] = 'True'
#         global print
#         print = print_new
#         return print_new
#     if isok == False:
#         os.environ['print_hook'] = 'True'
#         return print_old



import asyncio
from pyppeteer import launch

once = None
# launcher.DEFAULT_ARGS.remove("--enable-automation")
opt_chrome = {
    "devtools": True,
    'userDataDir': r'C:\Users\Administrator\Desktop\userdir',
    'executablePath': r'C:\Users\Administrator\AppData\Local\Google\Chrome\Application\chrome.exe',  # 浏览器安装路径   chrome://version
    'headless': False,
    'dumpio': True,  # 减少内存消耗   set_launch = {}
    # "ignoreDefaultArgs": ["--enable-automation"],
    'args': [
        # "--headless",
        "--mute-audio",  # 禁音
        '--no-sandbox',  # 关闭沙盒
        # '--disable-gpu',  # 禁用 GPU 硬件加速
        # '--log-level=3',  # 日志等级
        # '--disable-infobars',  # 禁用提示栏 --开启后页面加载不了
        '--window-size=1600,900',  # 屏幕大小
        '--window-position=100,100',  # 桌面位置
        "--user-agent=Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/78.0.3904.97 Safari/537.36",
    ],
}


class SPIDER:
    def __init__(self, *args, **kwargs):
        pass

    # 创建浏览器
    async def create_browser(self):
        # 初始化页面
        self.browser = await launch(opt_chrome)
        self.pages = await self.browser.pages()
        self.page = self.pages[0]
        await self.page.evaluate('''() => { Object.defineProperty(navigator, 'webdriver', {get: () => undefined})}''')  # evaluateOnNewDocument
        # await self.page.evaluate('''() => { Object.defineProperty(webdriver, 'webdriver', {get: () => false})}''')  # evaluateOnNewDocument

        # 反扒库
        self.web = self.pages[0]
        await self.web.setViewport({"width": 1920, "height": 1080})
        # await stealth(self.web)

        # 使用拦截器时会有问题
        # await self.page.setRequestInterception(True)  # 拦截
        # self.page.on('request', lambda req: asyncio.ensure_future(intercept(req)))
        return self

    # 页面跳转_设置cookies
    async def web_goto_set_cookie(self, url="", cookie={}):
        await self.web_set_cookies(cookie)  # 设置cookie

        # print("000                waitForSelector :", 111)
        await self.web.goto(url, {"timeout": 9 * 1000})
        # await self.web.waitForSelector("div", timeout=5000)
        # print("111                waitForSelector :", 111)
        await self.web.evaluate(""" document.title = 'web';  """)
        print("成功:页面跳转设置cookies--", "web_goto_set_cookie", url)
        return self
        pass

    # 页面跳转
    async def web_goto(self, url=""):
        await self.web.goto(url)
        await self.web.waitForSelector("html", timeout=10 * 1000)
        await self.web.evaluate(""" document.title = 'web';  """)
        return self

    # 等待css
    async def web_await_css(self, css="body", timeout=10 * 1000):
        try:
            await self.web.waitForSelector(css, timeout=timeout)
            await self.web.querySelector(css)
            return self
        except Exception as error:
            print("error web_await_css            :", css, str(error))

    # 点击元素
    async def web_await_click(self, css="body", timeout=10 * 1000):
        try:
            await self.web.waitForSelector(css, timeout=timeout)
            await self.web.click(css)
            # print("isok  web_await_click          :", css)
            return self
        except Exception as error:
            print("error web_await_click          :", css, str(error))

    # input上传文件
    async def web_await_input_files(self, css="body", files=[], timeout=10 * 1000):
        try:
            await self.web.waitForSelector(css, timeout=timeout)
            input_element = await self.web.querySelector(css)
            # await input_element.uploadFile(r"C:\Users\Administrator\Desktop\111.png", r"C:\Users\Administrator\Desktop\222.png")
            await input_element.uploadFile(*files)
        except Exception as error:
            print("error web_await_input_files          :", css, str(error))

    # input输入文字
    async def web_await_input_text(self, css="body", text="hello world", timeout=10 * 1000):
        try:
            await self.web.waitForSelector(css, timeout=11111)
            await self.web.click(css)
            await self.web.type(css, text), await asyncio.sleep(1)
        except Exception as error:
            print("error web_await_input_files          :", css, str(error))

    # 设置cookies
    async def web_set_cookies(self, cookies):
        if type(cookies) == dict:
            for key in cookies:
                # print("key            :", key)
                await self.web.setCookie({'name': key, 'value': cookies[key], 'domain': '.xiaohongshu.com'})

        if type(cookies) == list:
            for ele in cookies:
                await self.web.setCookie(ele)


async def spider_pyppeteer(x=1, y=1):
    """ EXPLAIN :   启动一个浏览器窗口
        EXAMPLE :
spider = await spider_pyppeteer()  #通常开发_浏览器位置,默认
spider = await __.spider.spider_pyppeteer(**dict( x=100, y=100))    #通常开发_浏览器位置,默认
spider = await __.spider.spider_pyppeteer(**dict(x=99999, y=99999)) #通常开发_浏览器位置,移动到屏幕外
page = spider.page
    """

    global once

    # if once is None:
    #     print("111---spider---实例初始化       :")
    #     once = await SPIDER().create_browser()
    #     return once
    # else:
    #     print("222---spider---实例已经存在     :")
    #     return once

    opt_chrome["args"] = [
        # "--headless",
        "--mute-audio",  # 禁音
        '--no-sandbox',  # 关闭沙盒
        '--disable-gpu',  # 禁用 GPU 硬件加速
        # '--log-level=3',  # 日志等级
        # '--disable-infobars',  # 禁用提示栏 --开启后页面加载不了
        '--window-size=1600,900',  # 屏幕大小
        f'--window-position={x},{y}',  # 桌面位置
        "--user-agent=Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/78.0.3904.97 Safari/537.36",
    ]
    once = await SPIDER().create_browser()
    print("成功:启动浏览器---spider                 位置", x, y)
    return once



import json
import shutil, os, pathlib
from astmain.fs.read import read

from types import SimpleNamespace
from collections import namedtuple


class DotDict(dict):
    __getattr__ = dict.__getitem__
    __setattr__ = dict.__setitem__

    def __call__(self, my_func_name, *args, **kwargs):
        print("my_func_name            :", my_func_name)
        result = self.__getitem__(my_func_name)(*args, **kwargs)

        return result


def func111():
    print("ctx            :", 111)


fff333 = DotDict({
    "aaa": 111,
    "read": read,
    "func111": func111
})
# python 帮我把代码   fff333.read().func111() 调用
# python 帮我把  data 转成  可以用  data.func111调用  也可以用 data["func111"]调用


if __name__ == '__main__':
    res1 = fff333.read(r"E:\AAA\xxx\ast_chrome\a08_驱动测试_cookie_打包_发布\astmain_py222\astmain\fs\aaa.txt")
    res2 = fff333["read"](r"E:\AAA\xxx\ast_chrome\a08_驱动测试_cookie_打包_发布\astmain_py222\astmain\fs\aaa.txt").func111()
    print("res1            :", res1)
    print("res2            :", res2)

    # res1 = fff333["read"](r"E:\AAA\xxx\ast_chrome\a08_驱动测试_cookie_打包_发布\astmain_py222\astmain\fs\aaa.txt")
    # print("res1            :", res1)

    # res2 = fff222(r"E:\AAA\xxx\ast_chrome\a08_驱动测试_cookie_打包_发布\astmain_py222\astmain\fs\maker_config.json").read("obj")
    # print("res2            :", type(res2), res2)



class class_base(type):

    def __str__(slef):
        # print("cls                 :" ,  slef   )
        aaa = slef
        return str(slef.name) + str(slef.age)
        pass


class MyClass(metaclass=class_base):
    name = 111
    age = 111
    aaa=1


print("111                 :", MyClass)
